#version 330 core

uniform vec3 lightColor = vec3(1.0, 0.95, 0.8);
uniform vec3 matSpecular = vec3(1.0, 1.0, 1.0); // Metall (chrom)
uniform float matShininess = 20.0f;

out vec4 color;

in vec3 normal;
in vec3 lightDir;
in vec3 colorVS;
in vec3 viewDir;


void main() {


    // normalize everything necessary
    vec3 N = normalize(normal);
    vec3 L = normalize(lightDir);

    // diffuse component
    float cosa = max(0.0, dot(N,L));
    vec3 diffuseTerm = colorVS * lightColor;

    color = vec4( diffuseTerm * cosa, 1.0);

    // specular component//
    vec3 V = normalize(viewDir);
    vec3 R = normalize(reflect(-L,N));
    float cosBeta = max(0.0, dot(R,V));
    float cosBetak = pow(cosBeta, matShininess);
    vec3 specularTerm = matSpecular * lightColor;

    color += vec4(specularTerm * cosBetak, 1.0);

    // ambient component //
    float ambientStrength = 0.2;
    vec3 ambientColor = colorVS; // this could also be a global ambient color
    vec3 ambientTerm = ambientColor * lightColor * ambientStrength;

    color += vec4(ambientTerm, 1.0);


}