#pragma once

#include "OpenGLWindow.h"
#include <ShaderProgram.h>
#include <memory>
#include <AssetManager.h>
#include "Transform.h"


static float roboterBodyHeadThiknessZ = 0.3;
static float roboterLegArmShoulderThiknessZ = 0.2;

static float animationTime = 0.0f;

static float rotationSpeed = 2.0f;

static float gap = 0.05;

struct ObjectBuffer {
    GLuint vaoID = 0;
    GLuint vboID = 0;
    GLuint iboID = 0;

    float* vertices = nullptr;
    GLsizei verticesLength = 0;
    GLsizeiptr sizeVerticesInBytes = 0;

    unsigned int* indices = nullptr;
    GLsizei indicesLength = 0;
    GLsizeiptr sizeOfIndicesInBytes = 0;

        ObjectBuffer() {};
        ObjectBuffer(const ObjectBuffer& other) {
        vaoID = other.vaoID;
        vboID = other.vboID;
        iboID = other.iboID;

        verticesLength = other.verticesLength;
        sizeVerticesInBytes = other.sizeVerticesInBytes;

        indicesLength = other.indicesLength;
        sizeOfIndicesInBytes = other.sizeOfIndicesInBytes;

        // Deep copy of vertex data
        if (other.vertices && verticesLength > 0) {
            vertices = new float[verticesLength];
            std::copy(other.vertices, other.vertices + verticesLength, vertices);
        }

        // Deep copy of index data
        if (other.indices && indicesLength > 0) {
            indices = new unsigned int[indicesLength];
            std::copy(other.indices, other.indices + indicesLength, indices);
        }
    }
};

class Scene
{
public:
    Scene(OpenGLWindow* window);
    ~Scene();

    bool init();
    void shutdown();
    void render(float dt);
    void update(float dt);
    OpenGLWindow* getWindow();

    void onKey(Key key, Action action, Modifier modifier);
    void onMouseMove(MousePosition mouseposition);
    void onMouseButton(MouseButton button, Action action, Modifier modifier);
    void onMouseScroll(double xscroll, double yscroll);
    void onFrameBufferResize(int width, int height);

private:
    OpenGLWindow* m_window;
    AssetManager m_assets;
    ShaderProgram* m_shader;
    ObjectBuffer obj;

    float x_eye,y_eye,z_eye;
    bool downPressed = false;

};

